package cn.newfeifan.mall.module.distri.service.ordercalc;

import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import cn.newfeifan.mall.module.distri.controller.admin.ordercalc.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.ordercalc.OrderCalcDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.pojo.PageParam;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;

import cn.newfeifan.mall.module.distri.dal.mysql.ordercalc.OrderCalcMapper;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;

/**
 * 订单产生数据计算 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class OrderCalcServiceImpl implements OrderCalcService {

    @Resource
    private OrderCalcMapper orderCalcMapper;

    @Override
    public Long createOrderCalc(OrderCalcSaveReqVO createReqVO) {
        // 插入
        OrderCalcDO orderCalc = BeanUtils.toBean(createReqVO, OrderCalcDO.class);
        orderCalcMapper.insert(orderCalc);
        // 返回
        return orderCalc.getId();
    }

    @Override
    public void updateOrderCalc(OrderCalcSaveReqVO updateReqVO) {
        // 校验存在
        validateOrderCalcExists(updateReqVO.getId());
        // 更新
        OrderCalcDO updateObj = BeanUtils.toBean(updateReqVO, OrderCalcDO.class);
        orderCalcMapper.updateById(updateObj);
    }

    @Override
    public void deleteOrderCalc(Long id) {
        // 校验存在
        validateOrderCalcExists(id);
        // 删除
        orderCalcMapper.deleteById(id);
    }

    private void validateOrderCalcExists(Long id) {
        if (orderCalcMapper.selectById(id) == null) {
            throw exception(ORDER_CALC_NOT_EXISTS);
        }
    }

    @Override
    public OrderCalcDO getOrderCalc(Long id) {
        return orderCalcMapper.selectById(id);
    }

    @Override
    public PageResult<OrderCalcDO> getOrderCalcPage(OrderCalcPageReqVO pageReqVO) {
        return orderCalcMapper.selectPage(pageReqVO);
    }

}