package cn.newfeifan.mall.module.distri.service.duser;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.duser.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.duser.DuserDO;
import cn.newfeifan.mall.module.distri.dal.mysql.duser.DuserMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link DuserServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(DuserServiceImpl.class)
public class DuserServiceImplTest extends BaseDbUnitTest {

    @Resource
    private DuserServiceImpl duserService;

    @Resource
    private DuserMapper duserMapper;

    @Test
    public void testCreateDuser_success() {
        // 准备参数
        DuserSaveReqVO createReqVO = randomPojo(DuserSaveReqVO.class).setId(null);

        // 调用
        Long duserId = duserService.createDuser(createReqVO);
        // 断言
        assertNotNull(duserId);
        // 校验记录的属性是否正确
        DuserDO duser = duserMapper.selectById(duserId);
        assertPojoEquals(createReqVO, duser, "id");
    }

    @Test
    public void testUpdateDuser_success() {
        // mock 数据
        DuserDO dbDuser = randomPojo(DuserDO.class);
        duserMapper.insert(dbDuser);// @Sql: 先插入出一条存在的数据
        // 准备参数
        DuserSaveReqVO updateReqVO = randomPojo(DuserSaveReqVO.class, o -> {
            o.setId(dbDuser.getId()); // 设置更新的 ID
        });

        // 调用
        duserService.updateDuser(updateReqVO);
        // 校验是否更新正确
        DuserDO duser = duserMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, duser);
    }

    @Test
    public void testUpdateDuser_notExists() {
        // 准备参数
        DuserSaveReqVO updateReqVO = randomPojo(DuserSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> duserService.updateDuser(updateReqVO), DUSER_NOT_EXISTS);
    }

    @Test
    public void testDeleteDuser_success() {
        // mock 数据
        DuserDO dbDuser = randomPojo(DuserDO.class);
        duserMapper.insert(dbDuser);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbDuser.getId();

        // 调用
        duserService.deleteDuser(id);
       // 校验数据不存在了
       assertNull(duserMapper.selectById(id));
    }

    @Test
    public void testDeleteDuser_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> duserService.deleteDuser(id), DUSER_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetDuserPage() {
       // mock 数据
       DuserDO dbDuser = randomPojo(DuserDO.class, o -> { // 等会查询到
           o.setUserId(null);
           o.setName(null);
           o.setMobile(null);
           o.setSource(null);
           o.setIntegralId(null);
           o.setSocialStatus(null);
           o.setSocialStatusPoint(null);
           o.setCategoryId(null);
           o.setCreateTime(null);
       });
       duserMapper.insert(dbDuser);
       // 测试 userId 不匹配
       duserMapper.insert(cloneIgnoreId(dbDuser, o -> o.setUserId(null)));
       // 测试 name 不匹配
       duserMapper.insert(cloneIgnoreId(dbDuser, o -> o.setName(null)));
       // 测试 mobile 不匹配
       duserMapper.insert(cloneIgnoreId(dbDuser, o -> o.setMobile(null)));
       // 测试 source 不匹配
       duserMapper.insert(cloneIgnoreId(dbDuser, o -> o.setSource(null)));
       // 测试 integralId 不匹配
       duserMapper.insert(cloneIgnoreId(dbDuser, o -> o.setIntegralId(null)));
       // 测试 socialStatus 不匹配
       duserMapper.insert(cloneIgnoreId(dbDuser, o -> o.setSocialStatus(null)));
       // 测试 socialStatusPoint 不匹配
       duserMapper.insert(cloneIgnoreId(dbDuser, o -> o.setSocialStatusPoint(null)));
       // 测试 categoryId 不匹配
       duserMapper.insert(cloneIgnoreId(dbDuser, o -> o.setCategoryId(null)));
       // 测试 createTime 不匹配
       duserMapper.insert(cloneIgnoreId(dbDuser, o -> o.setCreateTime(null)));
       // 准备参数
       DuserPageReqVO reqVO = new DuserPageReqVO();
       reqVO.setUserId(null);
       reqVO.setName(null);
       reqVO.setMobile(null);
       reqVO.setSource(null);
       reqVO.setIntegralId(null);
       reqVO.setSocialStatus(null);
       reqVO.setSocialStatusPoint(null);
       reqVO.setCategoryId(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<DuserDO> pageResult = duserService.getDuserPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbDuser, pageResult.getList().get(0));
    }

}